#!/usr/bin/python

import findrox; findrox.version(1,9,13)
import rox
from rox import g
HAVE_OGG = True
HAVE_MAD = True
try: import ogg.vorbis
except: HAVE_OGG = False
try: import mad
except: HAVE_MAD = False
import wave
import os, sys

app_name = 'Mp3Ogg2Wav'

def samp2msf(x):
	f=int(x/588)
	s=int(f/75)
	m=int(s/60)
#	return (m,s%60,f%75)
	return (m,s%60)

def samp2msf_str(x):
#	return '%02d:%02d.%02d' % (samp2msf(x))
	return '%02d:%02d' % (samp2msf(x))

def wav_fn(fn):
	return os.path.splitext(fn)[0]+'.wav'
	
def wave_file(fn):
	f = wav_fn(fn)
	if os.path.exists(f):
		if not rox.confirm('The file %s exists!\nDo you want to overwrite it?' % (f),g.STOCK_OK):
			return None
	dev = wave.open(f,'w')
	dev.setnchannels(2)
	dev.setsampwidth(2)
	dev.setframerate(44100)
	return dev

class Mp3Ogg2Wav(rox.Window):
	def __init__(self,files):
		rox.Window.__init__(self)
		
		self.pbar = g.ProgressBar()
		self.pbar.set_orientation(g.PROGRESS_LEFT_TO_RIGHT)
		self.pbar2 = g.ProgressBar()
		self.pbar2.set_orientation(g.PROGRESS_LEFT_TO_RIGHT)

		self.tv=g.TextView()
		self.tv.set_editable(False)
		self.tv.set_wrap_mode(g.WRAP_WORD)
		self.tv.get_buffer().connect('changed',self.auto_scroll)
		self.tv.set_editable(False)
		self.tv.set_cursor_visible(False)
		
		self.aborted = False

		swin = g.ScrolledWindow()
		swin.set_policy(g.POLICY_AUTOMATIC, g.POLICY_AUTOMATIC)
		swin.set_shadow_type(g.SHADOW_IN)
		swin.add(self.tv)
		
		butts = g.HButtonBox()
#		butts.set_layout(g.BUTTONBOX_SPREAD)
		butts.set_layout(g.BUTTONBOX_END)
		butts.set_border_width(4)
		butts.set_spacing(8)
		self.abort=rox.ButtonMixed(g.STOCK_NO,'Abort')
		self.abort.connect('clicked',self.abort_click)
		butts.add(self.abort)
		self.close=rox.ButtonMixed(g.STOCK_CLOSE,'Close')
		self.close.connect('clicked',self.close_handler)
		self.close.set_sensitive(False)
		butts.add(self.close)

		box = g.VBox(spacing=4)
		box.add(swin)
		box.pack_end(butts,False)
		box.pack_end(self.pbar,False)
		box.pack_end(self.pbar2,False)

		self.add(box)
		self.set_default_size(400,200)
		self.set_border_width(4)
		self.connect("delete-event",self.close_handler)
		self.set_title(app_name)
		self.show_all()

		nfiles = float(len(files))
		i=1.0
		i2=0
		
		try:
			for filename in files:
				self.pbar2.set_fraction(i/nfiles)
				self.pbar2.set_text('File %d of %d' % (i,nfiles))
				self.update_label(filename)
				ext = os.path.splitext(filename)[1].lower()
				r = False
				if ext == '.ogg': r = self.ogg2wav(filename)
				elif ext == '.mp3': r = self.mp32wav(filename)
				if self.aborted:
					self.add_line('ABORTED\nSkipping remaining files...\n')
					os.unlink(wav_fn(filename))
					break
				elif r: self.add_line('DONE\n'); i2+=1
				else: self.add_line('FAILED\n')
				i+=1.0
		except:
			rox.report_exception()
			
		self.add_line('All Done. Converted %d files\n' % (i2))
		self.abort_click()
		
	def abort_click(self,button=None):
		self.aborted=True
		self.abort.set_sensitive(False)
		self.close.set_sensitive(True)
		
	def close_handler(self,wid,ev=None):
		if not self.aborted:
			self.abort_click()
			return True
		if not ev: self.destroy()
		return False
		
	def auto_scroll(self,buf):
		self.tv.scroll_mark_onscreen(buf.get_insert())

	def add_line(self,txt):
		buf = self.tv.get_buffer()
		buf.insert(buf.get_end_iter(),txt)
				
	def update_label(self,fn):
		f = os.path.basename(fn)
		txt = 'Converting '+f+' to WAV... '
		self.add_line(txt)
#		self.set_title(f)

	def update_pbar(self,x,length):
		z=length/4.0
		y=z/x
		if y >= 0 and y <= 1: self.pbar.set_fraction(y)
		self.pbar.set_text(samp2msf_str(int(z))+' / '+samp2msf_str(int(x)))
		while g.events_pending(): g.main_iteration(False)
	
	def ogg2wav(self,filename):
		if not HAVE_OGG:
			rox.alert('You need libvorbis and its python bindings to handle OggVorbis files')
			return False
		dev = wave_file(filename)
		if not dev: return None

		vf = ogg.vorbis.VorbisFile(filename)
		x = float(vf.time_total(0)*44100.0)
		length = 0
		while not self.aborted:
			(buf, bytes, bit) = vf.read(4096)
			if not bytes: break
			dev.writeframesraw(buf[:bytes])
			length+=bytes
			self.update_pbar(x,length)
		dev.close()
		if self.aborted: return False
		return True

	def mp32wav(self,filename):
		if not HAVE_MAD:
			rox.alert('You need libmad and its python bindings to handle Mp3 files')
			return None
		dev = wave_file(filename)
		if not dev: return None

		mf = mad.MadFile(filename)
		x = float(mf.total_time()*44.1)
		length = 0
		while not self.aborted:
			buf = mf.read()
			if not buf: break
			dev.writeframesraw(buf)
			length += len(buf)
			self.update_pbar(x,length)
		dev.close()
		if self.aborted: return False
		return True
				
